$lines
     remark  program ANNUITY(input = CRT, output = CRT)

	comment
	   This is a program used to solve annuity based problems.
	   It assumes that payments/withdrawals occur at  the
	   beginning of the period.  The program was written in 
	   Structured Basic by Jeff Brown.  Version nos. for the
	   software are:

	      CP/M  2.2
	      S-Basic v5.4b, document revision 4.0 for KayPro II
	      Perfect Writer 1.20

	   Given one of three variables (Present/Future Value, Periodic
           Payment/Withdrawal, Years) it will determine the missing one's
	   value.  The basic formulas used to calculate these variables are:

	                                           (1 + r/n)**(mn) - 1
	      Generating the Annuity     V = P * -----------------------
	      (not contin. compound)              1 - (1 + r/n)**(-n/N)

	              or
	                                           e**(rm) - 1
	      Generating the Annuity     V = P * ---------------
	      (continuous compound)               1 - e**(-r/N)


	                                           1 - (1 + r/n)**(mn)
	      Withdrawing from Annuity   A = S * -----------------------
	      (not contin. compounding)           1 - (1 + r/n)**(-n/N)

	              or
	                                           1 - e**(rm)
	      Withdrawing from Annuity   A = S * ---------------
	      (continuous compounding)            1 - e**(-r/N)

	   where V is the future value, A is the present value, S is the
	   payment or withdrawal amount, r is the interest rate, m is the
	   number of years, n is the interest compounding period, and
	   N is the number of payments per year (period).  Note that e
	   stands for the natural logarithm.
	end of comment


	$constant  CRT = 0		;logical device, 0 = con:
	$constant  SEND.BEEP = 07	;ascii code for bell
	$constant  FALSE = 0		;logical statement false
	$constant  TRUE = 0FFFFH	;logical statement true
	$constant  DELAY = 2000		;loop count for screen refresh holdoff

	var  CALC.SELECTION		;calculation type selected
	     PERIOD.CHAR		;period character selected
	        = char
	var  PERIOD			;period value selected
	        = real.double
	var  MONEY.PRINT.FORMAT ;printing format of monetary output
	     RATE.PRINT.FORMAT	;printing format of percentage output
	     YEARS.PRINT.FORMAT	;printing format of number of years output
	        = string:20

	MONEY.PRINT.FORMAT = "$$##,###,###,###.##"
	RATE.PRINT.FORMAT = "####.##%"
	YEARS.PRINT.FORMAT = "####.## years\."


$list
	$include PRIMLIB HOME.AND.CLEAR.SCREEN
	$include PRIMLIB CHECK.VALID.CHAR
	$include PRIMLIB CHECK.VALID.REAL.NUMBER
	$include PRIMLIB CONVERT.LOWER.TO.UPPER
	$include PRIMLIB INVALID.INPUT
	$include PRIMLIB RETURN.TO.MAIN.MENU
$list


	procedure INTRO.TO.ANNUITY
	   comment
	      This module prints an introduction to the ANNUITY
	      Program.
	   end of comment

	   HOME.AND.CLEAR.SCREEN
	   print #CRT; "Welcome to the Annuity Problem Solver"
	   print #CRT; "How can I help you?"
	   print #CRT

	end of INTRO.TO.ANNUITY


	function GET.CALCULATION.SELECTION = char
	   comment
	      This module prints the menu selection available to the user
	      and then gets his response.
	   end of comment

	   var  THE.SELECTION	;user's character response to menu
	           = char
	   var  I		;loop index
	           = integer

	   text CRT,&
	If you can provide me with some information, I can
	calculate the following for you:

	   <F>uture value - knowing periodic payment, interest
	            rate, and number of years.
	   <P>ayment amount - knowing future value, interest rate,
	            and number of years.
	   <G>rowth period (years) - knowing periodic payment, future
	            value, and interest.

	   <I>initial value - knowing periodic withdrawal, interest
	            rate, and number of years.
	   <W>ithdrawal amount - knowing initial amount, interest
	            rate, number of years.
	   <D>istribution period (years) - knowing periodic withdrawal,
	            initial value, and interest rate.

	Just type your selection below or type <E> to exit.
	&
	   remark  turn off auto echo of user input
	   echo off

	   input1 #CRT; "Selection"; THE.SELECTION

	   remark  turn on auto echo of user input
	   echo on

	   THE.SELECTION = CONVERT.LOWER.TO.UPPER(THE.SELECTION)
	   if not CHECK.VALID.CHAR("FIPWGDEFIP",THE.SELECTION)  then  THE.SELECTION = 'N'
	   case THE.SELECTION of
	      'N':  begin
	               remark  invalid entry
	               HOME.AND.CLEAR.SCREEN
	               INVALID.INPUT 'M'
	            end of case 'N'
	      'F':  print #CRT; "Future Value"
	      'I':  print #CRT; "Initial Value"
	      'P':  print #CRT; "Payment Amount"
	      'W':  print #CRT; "Withdrawal Amount"
	      'G':  print #CRT; "Growth Period"
	      'D':  print #CRT; "Distribution Period"
	      'E':  print #CRT; "Exit"
	   end of case THE.SELECTION
	   if (THE.SELECTION <> 'N')  then
	      begin
	         for I = 1 to DELAY	remark  wait loop
	         next I
	         HOME.AND.CLEAR.SCREEN
	      end of if-then

	end = THE.SELECTION

	procedure PRINT.PERIOD(THE.SELECTION = char)
	   comment
	      This primitive translates the period character into the
	      full word and prints it.
	   end of comment

	   case THE.SELECTION of
	      'N':  remark  error, do nothing
	      'C':  print #CRT; "Continuous"
	      'D':  print #CRT; "Daily"
	      'W':  print #CRT; "Weekly"
	      'M':  print #CRT; "Monthly"
	      'Q':  print #CRT; "Quarterly"
	      'S':  print #CRT; "Semiannually"
	      'Y':  print #CRT; "Yearly"
	      'E':  print #CRT; "Exit"
	   end of case THE.SELECTION

	end of PRINT.PERIOD


	function GET.PERIOD.SELECTION = char
	   comment
	      This module gets the user's period selection.
	   end of comment

	   var  THE.SELECTION	;user's character response to menu
	           = char
	   var  I		;loop index
	           = integer

	   text CRT,&
	Which type of interest compounding period would you like?

	   <C>ontinuous
	   <D>aily
	   <W>eekly
	   <M>onthly
	   <Q>uarterly
	   <S>emiannually
	   <Y>early

	Type your selection below or type <E> to exit from this menu.
	&
	   remark  turn off auto echo of user input
	   echo off

	   input1 #CRT; "Selection"; THE.SELECTION

	   remark  turn on auto echo of user input
	   echo on

	   THE.SELECTION = CONVERT.LOWER.TO.UPPER(THE.SELECTION)
	   if not CHECK.VALID.CHAR("CDWMQSYECD",THE.SELECTION)  then
	         begin
	            remark  indicate error
	            THE.SELECTION = 'N'
	            HOME.AND.CLEAR.SCREEN
	            INVALID.INPUT 'M'
	         end
	      else
	         begin
	            PRINT.PERIOD THE.SELECTION
	            for I = 1 to DELAY	remark  wait loop
	            next I
	            HOME.AND.CLEAR.SCREEN
	         end of if-then-else
	
	end = THE.SELECTION


	function PERIOD.VALUE = real.double
	   comment
	      This primitive translates the period character into a numeric
	      value
	   end of comment

	   var  THE.VALUE		;period value
	           = real.double

	   case PERIOD.CHAR of
	      'N':  INVALID.INPUT 'G'
	      'C':  begin
	               remark  period is continuous, just set to zero
	               THE.VALUE = 0.0
	            end of case 'C'
	      'D':  begin
	               remark  period is daily
	               THE.VALUE = 365.0
	            end of case 'D'
	      'W':  begin
	               remark  period is weekly
	               THE.VALUE = 52.0
	            end of case 'W'
	      'M':  begin
	               remark  period is monthly
	               THE.VALUE = 12.0
	            end of case 'M'
	      'Q':  begin
	               remark  period is quarterly
	               THE.VALUE = 4.0
	            end of case 'Q'
	      'S':  begin
	               remark  period is semiannually
	               THE.VALUE = 2.0
	            end of case 'S'
	      'Y':  begin
	               remark  period is annually
	               THE.VALUE = 1.0
	            end of case 'Y'
	      'E':  INVALID.INPUT 'G'
	   end of case PERIOD.CHAR

	end = THE.VALUE


	function GET.REAL(PROMPT=char;LOWER.BOUND,UPPER.BOUND=real.double)=real.double
	   comment
	      This primitive gets a real number from the user and checks
	      is against a boundary set.  One exception is the 'T' case
	      where an whole real number is desired.
	   end of comment

	   var  THE.NUMBER	;number gotten from user
	           = real.double
	   var  AN.INTEGER	;used to get a whole number from the user
	        INPUT.IS.OK	;logical variable holding input check result
	           = integer

	   if not(CHECK.VALID.CHAR("FIRPWGDTFI",PROMPT))  then
	      INVALID.INPUT 'G'

	   repeat
	      begin
	         case PROMPT of
	            'F':  input1 #CRT; "        What is the Future Value"; THE.NUMBER
	            'I':  input1 #CRT; "        What is the Initial Value"; THE.NUMBER
	            'R':  input1 #CRT; "        What is the Interest Rate (i.e. 5, 8.5 ...)"; THE.NUMBER
	            'P':  input1 #CRT; "        What will the Payment Amount be"; THE.NUMBER
	            'W':  input1 #CRT; "        What will be the Withdrawal Amount"; THE.NUMBER
	            'G':  input1 #CRT; "        How many years will the annuity Grow"; THE.NUMBER
	            'D':  input1 #CRT; "        How many years will the annuity be Distributed"; THE.NUMBER
	            'T':  begin
	                     input1 #CRT; "        How many transactions per year (whole number 1 to 365)"; AN.INTEGER
	                     THE.NUMBER = AN.INTEGER
	                  end of 'T'
	         end of case PROMPT

	         INPUT.IS.OK = CHECK.VALID.REAL.NUMBER(LOWER.BOUND,UPPER.BOUND,THE.NUMBER)
	         if not INPUT.IS.OK  then  INVALID.INPUT 'R'
	      end
	   until  INPUT.IS.OK

	   print #CRT

	end = THE.NUMBER
   

	procedure FIND.FUTURE.VALUE
	   comment
	      This module calculates the future value, given the compounding
	      period, by inquiring as to the payment amount, the interest
	      rate, and the number of years it can grow.
	   end of comment

	   var  FUTURE.VALUE		;
	        PAYMENT.AMOUNT		;
	        INTEREST.RATE		;
	        NUMBER.OF.YEARS		;
	        TRANSACTIONS		;
	           = real.double

	   print #CRT; "Annuity Problem Solver invoking FUTURE VALUE formula"
	   print #CRT

	   print #CRT; "        The interest compounding period is ";
	   PRINT.PERIOD PERIOD.CHAR
	   print #CRT

	   PAYMENT.AMOUNT = GET.REAL('P',0.0,100000.00)
	   TRANSACTIONS = GET.REAL('T',1.0,365.0)
	   INTEREST.RATE = GET.REAL('R',0.001,1000.00) / 100.0
	   NUMBER.OF.YEARS = GET.REAL('G',0.0,1000.00)

	   if (PERIOD.CHAR <> 'C')  then  FUTURE.VALUE = PAYMENT.AMOUNT * (((1.0 + INTEREST.RATE / PERIOD)**(NUMBER.OF.YEARS * PERIOD) - 1.0) / (1.0 - (1.0 + INTEREST.RATE / PERIOD)**(-1.0 * PERIOD / TRANSACTIONS)))
	      else  FUTURE.VALUE = PAYMENT.AMOUNT * (exp(INTEREST.RATE * NUMBER.OF.YEARS) - 1.0) / (1.0 - exp(-1.0 * INTEREST.RATE / TRANSACTIONS))

	   print #CRT
	   print #CRT; "        The FUTURE VALUE is ";
	   print using MONEY.PRINT.FORMAT; #CRT; FUTURE.VALUE
	   print #CRT

	   RETURN.TO.MAIN.MENU

	end of FIND.FUTURE.VALUE


	procedure FIND.INITIAL.VALUE
	   comment
	      This module calculates the initial value, given the compounding
	      period, by inquiring as to the withdrawal amount, the interest
	      rate, and the number of years over which to distribute.
	   end of comment

	   var  WITHDRAWAL.AMOUNT	;
	        INITIAL.VALUE		;
	        INTEREST.RATE		;
	        NUMBER.OF.YEARS		;
	        TRANSACTIONS		;
	           = real.double

	   print #CRT; "Annuity Problem Solver invoking INITIAL VALUE formula"
	   print #CRT

	   print #CRT; "        The interest compounding period is ";
	   PRINT.PERIOD PERIOD.CHAR
	   print #CRT

	   WITHDRAWAL.AMOUNT = GET.REAL('W',0.0,100000.00)
	   TRANSACTIONS = GET.REAL('T',1.0,365.0)
	   INTEREST.RATE = GET.REAL('R',0.001,1000.00) / 100.0
	   NUMBER.OF.YEARS = GET.REAL('D',0.0,1000.00)

	   if (PERIOD.CHAR <> 'C')  then  INITIAL.VALUE = WITHDRAWAL.AMOUNT * ((1.0 - (1.0 + INTEREST.RATE / PERIOD)**(-1.0 * NUMBER.OF.YEARS * PERIOD)) / (1.0 - (1.0 + INTEREST.RATE / PERIOD)**(-1.0 * PERIOD / TRANSACTIONS)))
	      else  INITIAL.VALUE = WITHDRAWAL.AMOUNT * (1.0 - exp(-1.0 * INTEREST.RATE * NUMBER.OF.YEARS)) / (1.0 - exp(-1.0 * INTEREST.RATE / TRANSACTIONS))

	   print #CRT
	   print #CRT; "        The INITIAL VALUE is ";
	   print using MONEY.PRINT.FORMAT; #CRT; INITIAL.VALUE
	   print #CRT

	   RETURN.TO.MAIN.MENU

	end of FIND.INITIAL.VALUE


	procedure FIND.PAYMENT.AMOUNT
	   comment
	      This module calculates the payment amount, given the compounding
	      period, by inquiring as to the future value, the interest rate,
	      and the number of years to grow.
	   end of comment

	   var  FUTURE.VALUE		;
	        PAYMENT.AMOUNT		;
	        INTEREST.RATE		;
	        NUMBER.OF.YEARS		;
	        TRANSACTIONS		;
	           = real.double

	   print #CRT; "Annuity Problem Solver invoking PAYMENT AMOUNT formula"
	   print #CRT

	   print #CRT; "        The interest compounding period is ";
	   PRINT.PERIOD PERIOD.CHAR
	   print #CRT

	   TRANSACTIONS = GET.REAL('T',1.0,365.0)
	   FUTURE.VALUE = GET.REAL('F',0.0,100000000.00)
	   INTEREST.RATE = GET.REAL('R',0.001,1000.00) / 100.0
	   NUMBER.OF.YEARS = GET.REAL('G',0.001,1000.00)

	   if (PERIOD.CHAR <> 'C')  then  PAYMENT.AMOUNT = FUTURE.VALUE / (((1.0 + INTEREST.RATE / PERIOD)**(NUMBER.OF.YEARS * PERIOD) - 1.0) / (1.0 - (1.0 + INTEREST.RATE / PERIOD)**(-1.0 * PERIOD / TRANSACTIONS)))
	      else  PAYMENT.AMOUNT = FUTURE.VALUE / ((exp(INTEREST.RATE * NUMBER.OF.YEARS) - 1.0) / (1.0 - exp(-1.0 * INTEREST.RATE / TRANSACTIONS)))


	   print #CRT
	   print #CRT; "        The PAYMENT AMOUNT is ";
	   print using MONEY.PRINT.FORMAT; #CRT; PAYMENT.AMOUNT
	   print #CRT

	   RETURN.TO.MAIN.MENU

	end of FIND.PAYMENT.AMOUNT


	procedure FIND.WITHDRAWAL.AMOUNT
	   comment
	      This module calculates the withdrawal amount, given the
	      compounding period, by inquiring as to the initial value,
	      the interest rate, and the number of years over which to
	      distribute.
	   end of comment

	   var  INITIAL.VALUE		;
	        WITHDRAWAL.AMOUNT	;
	        INTEREST.RATE		;
	        NUMBER.OF.YEARS		;
	        TRANSACTIONS		;
	           = real.double

	   print #CRT; "Annuity Problem Solver invoking WITHDRAWAL AMOUNT formula"
	   print #CRT

	   print #CRT; "        The interest compounding period is ";
	   PRINT.PERIOD PERIOD.CHAR
	   print #CRT

	   TRANSACTIONS = GET.REAL('T',1.0,365.0)
	   INITIAL.VALUE = GET.REAL('I',0.0,100000000.00)
	   INTEREST.RATE = GET.REAL('R',0.001,1000.00) / 100.0
	   NUMBER.OF.YEARS = GET.REAL('D',0.001,1000.00)

	   if (PERIOD.CHAR <> 'C')  then  WITHDRAWAL.AMOUNT = INITIAL.VALUE / ((1.0 - (1.0 + INTEREST.RATE / PERIOD)**(-1.0 * NUMBER.OF.YEARS * PERIOD)) / (1.0 - (1.0 + INTEREST.RATE / PERIOD)**(-1.0 * PERIOD / TRANSACTIONS)))
	      else  WITHDRAWAL.AMOUNT = INITIAL.VALUE / ((1.0 - exp(-1.0 * INTEREST.RATE * NUMBER.OF.YEARS)) / (1.0 - exp(-1.0 * INTEREST.RATE / TRANSACTIONS)))

	   print #CRT
	   print #CRT; "        The WITHDRAWAL AMOUNT is ";
	   print using MONEY.PRINT.FORMAT; #CRT; WITHDRAWAL.AMOUNT
	   print #CRT

	   RETURN.TO.MAIN.MENU

	end of FIND.WITHDRAWAL.AMOUNT


	procedure FIND.GROWTH.PERIOD
	   comment
	      This module calculates the number of years needed for the
	      annuity to grow, given the period, by inquiring as to the
	      future value, the payment amount, and the interest rate.
	   end of comment

	   var  FUTURE.VALUE		;
	        PAYMENT.AMOUNT		;
	        INTEREST.RATE		;
	        NUMBER.OF.YEARS		;
	        TRANSACTIONS		;
	           = real.double

	   print #CRT; "Annuity Problem Solver invoking GROWTH PERIOD formula"
	   print #CRT

	   print #CRT; "        The interest compounding period is ";
	   PRINT.PERIOD PERIOD.CHAR
	   print #CRT

	   PAYMENT.AMOUNT = GET.REAL('P',0.001,100000.00)
	   TRANSACTIONS = GET.REAL('T',1.0,365.0)
	   FUTURE.VALUE = GET.REAL('F',0.001,100000000.00)
	   INTEREST.RATE = GET.REAL('R',0.001,1000.00) / 100.0

	   if (PERIOD.CHAR <> 'C')  then  NUMBER.OF.YEARS = log((FUTURE.VALUE / PAYMENT.AMOUNT) * (1.0 - (1.0 + INTEREST.RATE / PERIOD)**(-1.0 * PERIOD / TRANSACTIONS)) + 1.0) / (PERIOD * log(1.0 + INTEREST.RATE / PERIOD))
	      else  NUMBER.OF.YEARS = log((FUTURE.VALUE / PAYMENT.AMOUNT) * (1.0 - exp(-1.0 * INTEREST.RATE / TRANSACTIONS)) + 1.0) / INTEREST.RATE

	   print #CRT
	   print #CRT; "        The GROWTH PERIOD is ";
	   print using YEARS.PRINT.FORMAT; #CRT; NUMBER.OF.YEARS
	   print #CRT

	   RETURN.TO.MAIN.MENU

	end of FIND.GROWTH.PERIOD


	procedure FIND.DISTRIBUTION.PERIOD
	   comment
	      This module calculates the number of years a distribution
	      will last, given the period, by inquiring as to the initial
	      value, the withdrawal amount, and the interest rate.
	   end of comment

	   var  WITHDRAWAL.AMOUNT	;
	        INITIAL.VALUE		;
	        INTEREST.RATE		;
	        NUMBER.OF.YEARS		;
	        TRANSACTIONS		;
	           = real.double

	   print #CRT; "Annuity Problem Solver invoking DISTRIBUTION PERIOD formula"
	   print #CRT

	   print #CRT; "        The interest compounding period is ";
	   PRINT.PERIOD PERIOD.CHAR
	   print #CRT

	   WITHDRAWAL.AMOUNT = GET.REAL('W',0.001,100000.00)
	   TRANSACTIONS = GET.REAL('T',1.0,365.0)
	   INITIAL.VALUE = GET.REAL('I',0.001,100000000.00)
	   INTEREST.RATE = GET.REAL('R',0.001,1000.00) / 100.0

	   if (PERIOD.CHAR <> 'C')  then  NUMBER.OF.YEARS = -1.0 * log(1.0 - (INITIAL.VALUE / WITHDRAWAL.AMOUNT) * (1.0 - (1.0 + INTEREST.RATE / PERIOD)**(-1.0 * PERIOD / TRANSACTIONS))) / (PERIOD * log(1.0 + INTEREST.RATE / PERIOD))
	      else  NUMBER.OF.YEARS = -1.0 * log(1.0 - (INITIAL.VALUE / WITHDRAWAL.AMOUNT) * (1.0 - exp(-1.0 * INTEREST.RATE / TRANSACTIONS))) / INTEREST.RATE

	   print #CRT
	   print #CRT; "        The DISTRIBUTION PERIOD is ";
	   print using YEARS.PRINT.FORMAT; #CRT; NUMBER.OF.YEARS
	   print #CRT

	   RETURN.TO.MAIN.MENU

	end of FIND.DISTRIBUTION.PERIOD


	procedure TERMINATE.ANNUITY
	   comment
	      This module closes off and exits from the ANNUITY
	      Program.
	   end of comment

	   print #CRT; "Annuity Problem Solver saying goodbye!"

	end of TERMINATE.ANNUITY


	begin
	   INTRO.TO.ANNUITY
	   repeat
	      begin
	         CALC.SELECTION = GET.CALCULATION.SELECTION
	         if ((CALC.SELECTION <> 'N') and (CALC.SELECTION <> 'E')) then
	            begin
	               repeat
	                  PERIOD.CHAR = GET.PERIOD.SELECTION
	               until PERIOD.CHAR <> 'N'
	               if ( PERIOD.CHAR = 'E' )  then  CALC.SELECTION = 'N'
	                  else  PERIOD = PERIOD.VALUE
	            end
	         case CALC.SELECTION of
	            'N':  remark  do nothing
	            'F':  FIND.FUTURE.VALUE
	            'I':  FIND.INITIAL.VALUE
	            'P':  FIND.PAYMENT.AMOUNT
	            'W':  FIND.WITHDRAWAL.AMOUNT
	            'G':  FIND.GROWTH.PERIOD
	            'D':  FIND.DISTRIBUTION.PERIOD
	            'E':  remark  termination
	         end of case CALC.SELECTION
	      end 
	   until CALC.SELECTION = 'E'
	   TERMINATE.ANNUITY
	   stop
	end
nt #CRT; "Annuity Problem Solver saying goodbye!"

	end of TERMINATE.ANNUITY


	begin