/*
%CC1 $1.C -X
*/
/*********************************************************************
*                               WILDEXP                              *
*********************************************************************/
/*
	WILDEXP.C 	v2.0	12/26/83
	BDS C Command-line Wild-card expansion utility
	
	Revised by Gene Mallory
	Written by Leor Zolman

	Lets ambiguous file names appear on the command line to C programs,
	automatically expanding the parameter list to contain all files that
	fit the afn's. Returns ERROR if something went wrong; usually, this
	is due to MAXITEMS being set too low; the "MAXITEMS" defined constant
	sets the maximum number of matches that WILDEXP can handle. If you
	need to be able to process directories with more entries than this,
	be sure to modify MAXITEMS to be bigger than the largest expected
	directory size.

	An afn preceded by a "!" causes all names matching the given afn to
	be EXCLUDED from the resulting expansion list. Thus, to yield a
	command line containing all files except "COM" files, you'd say:

		A>progname !*.com <cr>

	Another example: to get all files on B: except .C files, say:

		A>prognam b:*.* !b:*.c <cr>

	When giving a "!" afn, "*" chars in the string matches to the end of
	either the filename or extension, just like CP/M, but "?" chars match
	ONE and ONLY ONE character in either the filename or extension.

	To use WILDEXP, begin your "main" function as follows:

	---------------------------------------------
	main(argc,argv)
	char **argv;
	{
	...				   /* local declarations  */
		dioinit(&argc,argv);		   /* if using DIO	  */
		if (wildexp(&argc,&argv) == ERROR) /* 1st state. in prog. */
		   exit(puts("Wildexp overflow"));
		...
	---------------------------------------------

	and link WILDEXP.CRL in with your program. That's all there is to
	it; note that "wildexp" uses the "sbrk" function to obtain storage,
	so don't go playing around with memory that is outside of the
	external or stack areas unless you obtain the memory through "sbrk"
	or "alloc" calls.
*/

#INCLUDE "BDSCIO.H"

#DEFINE MAXITEMS 300 /* max no. of items after expansion */

#DEFINE SEARCH_FIRST 17 /* BDOS calls */
#DEFINE SEARCH_NEXT 18
#DEFINE GET_DRIVE 25
#DEFINE GET_USER 32
#DEFINE SET_DISK 14
#DEFINE SET_USER 32
#DEFINE SET_DMA 26

wildexp(argcptr, argvptr)
INT *argcptr;  	/* pointer to argc */
CHAR ***argvptr;  	/* pointer to argv */
  BEGIN
    INT newargc;  	/* new argc */
    CHAR **newargv;  	/* new argv */
    CHAR **oldargv;  	/* old argv */
    INT oldargc;  	/* old argc */
    CHAR fcb[36];  	/* fcb used for search for first/next calls */
    CHAR dmapos;  	/* value returned by search calls */
    CHAR first_time;  	/* used in search routine */
    CHAR tmpfn[20];  	/* temporary file name */
    CHAR tmpstring[20];  	/* working string */
    CHAR *notfns[20];  	/* list of !<afn> entries */
    INT notcount;  	/* count of entries in notfns */
    CHAR cur_drive;  	/* current drive number */
    CHAR cur_user;  	/* current user number */
    CHAR usrno;  	/* this file user number */
    CHAR driveno;  	/* this file drive number */
    CHAR buffer[128];  	/* directory buffer */
    INT i,j,k;

    cur_drive = bdos(GET_DRIVE);
    cur_user = bdos(GET_USER,0xff);
    bdos(SET_DMA,buffer);

    oldargv = *argvptr;
    oldargc = *argcptr;
    newargc = 1;
    notcount = 0;

    IF ((newargv = sbrk(MAXITEMS*2 + 2)) == ERROR) /* get newargv space  */
    error("WILDEXP: Out of RAM space");
    IF (oldargv[1][0] == '!') /* If first name is a !name */
      THEN
	notfns[notcount++] = &oldargv[1][1];
	oldargv[1] = "*.*";
      ENDIF
    FOR (i = 1; i < oldargc; i++)
      LOOP
	IF (oldargv[i][0] == '!') /* add not name to not list */
	  THEN
	    notfns[notcount++] = &oldargv[i][1];
	  ENDIF

	ELSE /* Just name, search file system */
	  THEN
	    fsplit(oldargv[i],tmpfn,&usrno,&driveno,cur_drive,cur_user);
	    setfcb(fcb,tmpfn);
	    bdos(SET_DISK,driveno);
	    bdos(SET_USER,usrno);

	    first_time = TRUE;
	    WHILE (1) 
	      LOOP
		/* find all matching files */
		dmapos = bdos((first_time?SEARCH_FIRST:SEARCH_NEXT), fcb);
		first_time = FALSE;
		IF (dmapos == 255) 
		  THEN
		    BREAK;
		  ENDIF
		/* strip name out of fcb */
		hackname(tmpfn,(buffer+dmapos*32)
		,cur_drive,cur_user,driveno,usrno);
		/* get space for name */
		IF ((newargv[newargc] = sbrk(20)) == ERROR)
		  THEN
		    error("WILDEXP: Out of RAM space");
		  ENDIF
		IF (newargc >= MAXITEMS)
		  THEN
		    error("WILDEXP: Out of RAM space");
		  ENDIF
		/* construct file name for new argv */
		strcpy(newargv[newargc++], tmpfn);
	      ENDLOOP
	    bdos(SET_DISK,cur_drive);  	/* restore to current drive */
	    BDOS(SET_USER,cur_user);  	/* restore to current user */
	  ENDIF
      ENDLOOP
/*  Now eliminate all the NOT names  */ 
    FOR (i = 0; i < notcount; i++)
    FOR (j = 1; j < newargc; j++)
    WHILE (match(notfns[i],newargv[j],cur_drive,cur_user))
      LOOP
	IF(j == --newargc)
	  THEN
	    BREAK;
	  ENDIF
	FOR (k = j; k < newargc; k++)
	  LOOP
	    newargv[k] = newargv[k+1];
	  ENDLOOP
      ENDLOOP

    *argcptr = newargc;  	/* Replace argc and argv */
    *argvptr = newargv;  	/* with new values */ 
    RETURN 0;
  END

hackname(dest,source,cd,cu,nd,nu)
CHAR *dest, *source;
CHAR cd,cu,nd,nu;
  BEGIN
    INT i,j;

    j = 0;
    IF (cu != nu) /* user no if any */
      THEN
	dest[j++] = nu/10 + '0';
	dest[j++] = nu%10 + '0';
	dest[j++] = '/';
      ENDIF
    IF (cd != nd) /* drive name if any */
      THEN
	dest[j++] = nd+'A';
	dest[j++] = ':';
      ENDIF
    FOR (i = 1; i < 9; i++) /* file name */
      LOOP
	IF (source[i] == ' ') 
	  THEN
	    BREAK;
	  ENDIF
	dest[j++] = source[i];
      ENDLOOP
    IF (source[9] != ' ')
      THEN
	dest[j++] = '.';
      ENDIF

    FOR (i = 9; i < 12; i++) /* file name tag */
      LOOP
	IF (source[i] == ' ') 
	  THEN
	    BREAK;
	  ENDIF
	dest[j++] = source[i] & 0x7F;
      ENDLOOP
    dest[j] = '\0';
    RETURN dest;
  END

INT match(wildnam, filnam, cur_drive, cur_user)
CHAR *wildnam, *filnam, cur_drive, cur_user;
  BEGIN
    CHAR c,u1,d1,u2,d2,fname1[20],fname2[20],*f1ptr,*f2ptr;
    fsplit(wildnam,fname1,&u1,&d1,cur_drive,cur_user);
    fsplit(filnam,fname2,&u2,&d2,cur_drive,cur_user);
    if (u1 != u2) return FALSE;
    if (d1 != d2) return FALSE;
    f1ptr = fname1;
    f2ptr = fname2;

    WHILE (c = *f1ptr++)
      LOOP
	IF (c == '?')
	IF ((c = *f2ptr++) && c != '.')
	CONTINUE;
	ELSE
	RETURN FALSE;
	ELSE IF (c == '*')
	  THEN
	    WHILE (c = *f1ptr)
	      LOOP
		f1ptr++;
		IF (c == '.') BREAK;
	      ENDLOOP
	    WHILE (c = *f2ptr)
	      LOOP
		f2ptr++;
		IF (c == '.') BREAK;
	      ENDLOOP
	  ENDIF
	ELSE IF (c == *f2ptr++)
	CONTINUE;
	ELSE RETURN FALSE;
      ENDLOOP

    IF (!*f2ptr)
    RETURN TRUE;
    ELSE
    RETURN FALSE;
  END


FUNCTION fsplit(bigname,fname,userptr,driveptr,cur_drive,cur_user)
CHAR *bigname, *fname, *userptr, *driveptr, cur_drive, cur_user;
  BEGIN
    IF ((isdigit(bigname[0]) && (bigname[1] == '/'))
    ||(isdigit(bigname[0]) && isdigit(bigname[1]) && (bigname[2] == '/')))
      THEN
	*userptr = atoi(bigname);
	WHILE (*bigname++ != '/') ;
      ENDIF
    ELSE
      THEN
	*userptr = cur_user;
      ENDIF
    IF (isalpha(bigname[0]) && (bigname[1] == ':'))
      THEN
	*driveptr = toupper(bigname[0])-'A';
	bigname +=2;
      ENDIF
    ELSE
      THEN
	*driveptr = cur_drive;
      ENDIF
    strcpy(fname, bigname);
  END
